; ---------------------STANDARD HEADER-------------------
;
; Tell the AS assembler we want to code for NGPC
;
        cpu 93C141
        maxmode on
        include "stddef96.inc"


        include "hardware.inc"  ; Include hardware defines
        include "system.inc"    ; Include System Call defines
        

START_OF_RAM    EVAL    _MAINRAM
	RAMDB VBCounter


HEADER_OFS      EQU     0200000H

;
; Standard cartridge header
;
    org    HEADER_OFS
    db     " LICENSED BY SNK CORPORATION"    ; 28 bytes license string
    dd     _start                            ; Program Counter
    dw     0                                 ; Catalog number
    db     0                                 ; Sub catalog number
    db     10h                               ; colour or b+w (10h = colour)
;           123456789012
    db     "            "                    ; Game name (12 bytes)
    dd     0,0,0,0                           ; padding - reserve for future use

; User Interrupt Vectors

UserIntVect:
        dd      nada            ; Software Interrupt (SWI 3)
        dd      nada            ; Software Interrupt (SWI 4)
        dd      nada            ; Software Interrupt (SWI 5)
        dd      nada            ; Software Interrupt (SWI 6)
        dd      nada            ; RTC Alarm Interrupt
        dd      VBlankInt       ; Vertical Blanking Interrupt
        dd      nada            ; Interrupt from Z80
        dd      nada            ; Timer Interrupt (8 bit timer 0)
        dd      nada            ; Timer Interrupt (8 bit timer 1)
        dd      nada            ; Timer Interrupt (8 bit timer 2)
        dd      nada            ; Timer Interrupt (8 bit timer 3)
        dd      nada            ; Serial Transmission Interrupt
        dd      nada            ; Serial Reception Interrupt
        dd      nada            ; (Reserved)
        dd      nada            ; End Micro DMA Int (MicroDMA 0)
        dd      nada            ; End Micro DMA Int (MicroDMA 1)
        dd      nada            ; End Micro DMA Int (MicroDMA 2)
        dd      nada            ; End Micro DMA Int (MicroDMA 3)

nada:   reti

; *** Start of User Code ***

_start:
        calr    OS_VERSION      ; Initialize NGP or NGPC mode

        set     6,(rUSERA)      ; User Answer

; Install User Interrupt Vectors

        lda     xix,(UserIntVect)
        lda     xiy,(rSWI3)
        ld      b,18
UIVloop:
        ld      xwa,(xix+)
        ld      (xiy+),xwa
        djnz    b,UIVloop

; set up screen size 160x152
    ld   (08002h),0
    ld   (08003h),0
    ld   (08004h),0a0h
    ld   (08005h),98h

    ldw (_BGCPAL), 0
    ld (8118h), 80h

; enable interrupts
    ei


; ---------------------END OF STANDARD HEADER-------------------


; initialisation code

    ld c, 2                   ; I have 2 sound effects
    ld xix, MySoundEffects    ; and here they are.
    calr SFX_Initialise


; the main loop

_main:

    ld a, (rSL)               ; read joystick
    
    bit PADB_A, a             ; play our first sound
    jr z, Label_1
    ld c,1
    call SFX_PlaySound

Label_1:
    bit PADB_B, a             ; play our second sound
    jr z, Label_2
    ld c, 2
    call SFX_PlaySound

Label_2:




; User shutdown? (Power off pressed?)

        cp      (rUSERS),0
        jp      z,_main      ; no

; Power off NGP

        ld      rw3,VECT_SHUTDOWN
        calr    SYSTEM_CALL

_done:   jr      _done



VBlankInt:
    ; Update Watch Dog Timer to prevent CPU reset
    ld      (rWDCR),WD_CLR

    ; increment a counter
    inc (VBCOUNTER)
    call SFX_VBTimer
    reti


    include "sfxlib.inc"


SYSTEM_CALL:
        SystemCallCode
OS_VERSION:
        OsVersionCode


MySoundEffects:
      ; a chirpy bird on channel 1
      db 1,40h,0,80h,0,20h,0,1,1,44h,0
      db 82h,0,0fh,0,0,0,0,0

      ; a siren on channel 2
      db 2,60h,0,60h,0,8,0,1,2,40h,0
      db 80h,0,0fh,0,0,0,0,0

    ; make the cartridge size correct
    org HEADER_OFS + 07FFFh
    db 0FFh

