; ****************************************************
;
; SFX Library by Ivan Mackintosh 21-7-01
;
; To use this library include towards the end of
; your main .asm file and make the following changes
; to your code:
;
; Call SFX_VBTimer from your VBInterrupt code
; 
; Call SFX_Initialise during the initialisation of 
; you program, passing in how many sound effects
; you have and a pointer to the first one.
;
; Call SFX_PlaySound passing the index of your sound
; that you want to play (in the range of 1 to 31)
; 
; Calling SFX_Quiet turns off all currently playing
; sound effects
;
; ****************************************************


; ------------------------------------------
; SFX_Initialise
; Installs the sound driver into the z80 and
; loads the preset sound effects
;
; Inputs: 
;    xix - pointer to sound effects buffer
;    c   - number of sound effects
; 
; ------------------------------------------
SFX_Initialise:
   push a
   push xhl
   calr InstallSoundDriver

   ld a, 1
   ld xhl, xix

SFX_InstallNext:
   cp c, 0
   jr z, SFX_InitialiseEnd
   calr LoadSFXPreset
   inc a
   add xhl, SFXSoundEffectSize
   dec c
   jr SFX_InstallNext
   
SFX_InitialiseEnd:
   pop xhl
   pop a
   ret



; ------------------------------------------
; SFX_PlaySound
; Plays one of the predefined sounds passed
; in when SFX_Initialise was called
;
; Inputs: 
;    c - the sound to play (range = 1 to 31)
; 
; ------------------------------------------
SFX_PlaySound:
   ld (SFXPlayPreset), c
   ret



; ------------------------------------------
; SFX_Quiet
; Turns off all currently playing sounds
;
; Inputs: 
;    None
; 
; ------------------------------------------
SFX_Quiet:
   ld (SFXStopAll), 1
   ret



; ------------------------------------------
; SFX_VBTimer
; This must be called periodically from the
; TLCS VB Interrupt code.
;
; Inputs: 
;    None
; 
; ------------------------------------------
SFX_VBTimer:
   inc (SfxPulse)
   ret



; --------------------------------------------------------
; Below are the private functions that you don't need to 
; worry about. The public functions are all above this
; comment
; --------------------------------------------------------

SFXSoundEffectSize   equ   19
SFXPulse             equ	7010h
SFXPlayBuffer        equ	7011h
SFXLoadPreset        equ	7012h
SFXPlayPreset        equ	7013h
SFXStopAll           equ	7014h
SFXBuffer            equ	7020h

InstallSoundDriver:
   push bc
   push xde
   push xhl
	ldw (00b8h), 0aaaah ; stop the z80 and sound chip

	; copy the driver into memory
	ld bc, SoundDriverEnd - SoundDriver
	ld xde, 7000h ; where the z80 shared memory is
	ld xhl, SoundDriver
	ldir (xde+), (xhl+)

	ldw (00b8h), 05555h ; start the z80 and sound chip
   pop xhl
   pop xde
   pop bc
	ret
	
SoundDriver:
	binclude "sfxdriv.z80"
SoundDriverEnd:

; i/ps a = number, xhl = where sfx stored
LoadSFXPreset:
    push a
    push bc
    push xde
    push xhl
    ld bc, SFXSoundEffectSize
    ld xde, SFXBuffer
    ldir (xde+),(xhl+)
    ld (SFXLoadPreset), a
; wait for sfx to be processed
LdSFXPreset_Wait: 
    ld a, (SFXLoadPreset) 
    cp a, 0
    jr nz, LdSFXPreset_Wait
    pop xhl
    pop xde
    pop bc
    pop a
    ret
