; Tell the AS assembler we want to code for NGPC
;
        cpu 93C141
        maxmode on
        include "stddef96.inc"


        include "hardware.inc"  ; Include hardware defines
        include "system.inc"    ; Include System Call defines
        

START_OF_RAM    EVAL    _MAINRAM
		RAMDB		VBCounter
		RAMDB		PlayState


HEADER_OFS      EQU     0200000H


;
; Standard cartridge header
;
    org    HEADER_OFS
    db     " LICENSED BY SNK CORPORATION"    ; 28 bytes license string
    dd     _start                            ; Program Counter
    dw     0                                 ; Catalog number
    db     0                                 ; Sub catalog number
    db     10h                               ; colour or b+w (10h = colour)
;           123456789012
    db     "SNDTOOL TEST"                    ; Game name (12 bytes)
    dd     0,0,0,0                           ; padding - reserve for future use

; User Interrupt Vectors

UserIntVect:
        dd      nada            ; Software Interrupt (SWI 3)
        dd      nada            ; Software Interrupt (SWI 4)
        dd      nada            ; Software Interrupt (SWI 5)
        dd      nada            ; Software Interrupt (SWI 6)
        dd      nada            ; RTC Alarm Interrupt
        dd      VBlankInt       ; Vertical Blanking Interrupt
        dd      nada            ; Interrupt from Z80
        dd      nada            ; Timer Interrupt (8 bit timer 0)
        dd      nada            ; Timer Interrupt (8 bit timer 1)
        dd      nada            ; Timer Interrupt (8 bit timer 2)
        dd      nada            ; Timer Interrupt (8 bit timer 3)
        dd      nada            ; Serial Transmission Interrupt
        dd      nada            ; Serial Reception Interrupt
        dd      nada            ; (Reserved)
        dd      nada            ; End Micro DMA Int (MicroDMA 0)
        dd      nada            ; End Micro DMA Int (MicroDMA 1)
        dd      nada            ; End Micro DMA Int (MicroDMA 2)
        dd      nada            ; End Micro DMA Int (MicroDMA 3)

nada:   reti

; *** Start of User Code ***

_start:
        calr    OS_VERSION      ; Initialize NGP or NGPC mode

        set     6,(rUSERA)      ; User Answer

; Install User Interrupt Vectors

        lda     xix,(UserIntVect)
        lda     xiy,(rSWI3)
        ld      b,18
UIVloop:
        ld      xwa,(xix+)
        ld      (xiy+),xwa
        djnz    b,UIVloop

; set up screen size 160x152
    ld   (08002h),0
    ld   (08003h),0
    ld   (08004h),0a0h
    ld   (08005h),98h

    ldw (_BGCPAL), 0
    ld (8118h), 80h

    calr InitialiseMemory
	 
	 ld (PlayState), 0
	 
; copy internal charset to character RAM
        ld      ra3,3
        ld      rw3,VECT_SYSFONTSET
        calr    SYSTEM_CALL

; copy palette to character palette RAM
    ld bc, (ENDPALETTE-PALETTE)/2
    ld xde, _SCR1PAL
    ld xhl, PALETTE
    ldirw (xde+),(xhl+)



; enable interrupts
    ei

	 ; copy group
	 ldw bc, (TEST_LEN)
	 ld xhl, TEST_DATA
	 calr NeoTracker_SendGroup
	 calr But_A_Play

_main:
    ld a, (rSL)
  
	 cp a, PADB_UP
	 jp nz, ButtonA

_JoyStick_End:
    ld w, 5
    calr vbWait


; User shutdown? (Power off pressed?)

        cp      (rUSERS),0
        jp      z,_main      ; no

; Power off NGP

        ld      rw3,VECT_SHUTDOWN
        calr    SYSTEM_CALL

_done:   jr      _done




ButtonA:
	; first wait for button release
	ld a, (rSL)
	cp a, 0
	jr nz, ButtonA

	cp (PlayState), 0
	jr z, But_A_Play
	jr But_A_Silence


strPlay  db "PLAYING", 0
strBlank db "       ", 0

But_A_Play:
	push xwa
	push xde
	push xhl
	ld (PlayState), 1
	ld xde, strPlay
	ldw hl, 0603h
	calr RenderString
	ld a, ARKANOID
	calr NeoTracker_PlayMusic
	pop xhl
	pop xde
	pop xwa
	jp _JoyStick_End

But_A_Silence:
	push xde
	push xhl
	ld (PlayState), 0
	ld (REG_Z80_COMM), 0ffh
	ld (REG_Z80_NMI), 1
	ld xde, strBlank
	ldw hl, 0603h
	calr RenderString
	pop xhl
	pop xde
	jp _JoyStick_End




; ---------------------------------------------------------------
; Initialise Memory before game starts
;
; ---------------------------------------------------------------
InitialiseMemory:
; Clear Sprite Attribute Memory

        ld      wa,0
        ld      b,64
        ld      xix,_SPRRAM
SAMloop:
        ld      (xix+),wa
        ld      (xix+),wa
        djnz    b,SAMloop

; Clear RAM
		  ld wa, 0
		  ld bc, 800h
		  ld xix, _MAINRAM
RAMloop:		  
		  ld (xix+), wa
		  djnz bc, RAMloop

; Clear Scroll 1 & 2 VRAM with 0
ClearScreen:
        ld      xhl,_SCR1RAM
        ld      wa, 0
        ld      (xhl),wa
        ld      xde,xhl
        inc     xde
        inc     xde
        ld      bc,800h-1
        ldirw
        ret



; ---------------------------------------------------------------
; 
; DrawCharFront
;
; Draws a character on the foreground plane
; inputs: wa = char, hl = xy
;
; ---------------------------------------------------------------
DrawCharFront
    push xhl
    push xde
    push xbc
    ld xde, _SCR1RAM
    ld c, h
    ld b, 0
    mul xbc, 2
    add de, bc
    ld h, 0
    mul xhl, 40h
    add de, hl
    ld (xde), a
    add xde, 1
    ld (xde), w
    pop xbc
    pop xde
    pop xhl
    ret


; ---------------------------------------------------------------
; Draws text on the screen (front plane)
; inputs: xde=pointer to string
;         hl = xy
; string followed by 0
; ---------------------------------------------------------------
RenderString:
    push xwa
	 push xde
    push xhl
    ld xwa, 0
rsAgain:
    ld a, (xde+)
	 cp a, 0
	 jr z, rsEnd
    calr DrawCharFront
    inc h
    jr nz, rsAgain
rsEnd:
    pop xhl
	 pop xde
    pop xwa
    ret




VBlankInt:
    ; Update Watch Dog Timer to prevent CPU reset
    ld      (rWDCR),WD_CLR

    ; increment a counter
    inc (VBCOUNTER)

    reti

; ---------------------------------------------------------------
;
; vbWait
; Waits for n vertical blank interrupts
; inputs: w = number of VBIs to wait for
;
; ---------------------------------------------------------------
vbWait:
   push xwa
   push xbc
   ld (VBCOUNTER), 0
   ld c, w
vbw1:
   ld w, c
   ld a, (VBCOUNTER)
   sub w, a
   jr nc, vbw1
   pop xbc
   pop xwa
   ret




PALETTE
	dw 00000h, 00fffh, 00fffh, 00fffh
ENDPALETTE

    include "ntrkrlib.inc"
	 include "test.inc"

SYSTEM_CALL:
        SystemCallCode
OS_VERSION:
        OsVersionCode

    ; make the cartridge size correct
    org HEADER_OFS + 07FFFh
    db 0FFh
