; Tell the AS assembler we want to code for NGPC
;
        cpu 93C141
        maxmode on
        include "stddef96.inc"


        include "hardware.inc"  ; Include hardware defines
        include "system.inc"    ; Include System Call defines
        

START_OF_RAM    EVAL    _MAINRAM
	RAMDB VBCounter
	RAMDB CurPos
	RAMDW Channel
	RAMDW Length
	RAMDW Repeat
	RAMDW Init_Tone
	RAMDW Tone_Step
	RAMDW Tone_Speed
	RAMDW Tone_OWB
	RAMDW Tone_LLimit
	RAMDW Tone_ULimit
	RAMDW Init_Vol
	RAMDW Vol_Step
	RAMDW Vol_Speed
	RAMDW Vol_OWB
	RAMDW Vol_LLimit
	RAMDW Vol_ULimit

MAX_VARIABLES	EQU	15

HEADER_OFS      EQU     0200000H


SFXPulse	equ	7010h
SFXPlayBuffer	equ	7011h
SFXLoadPreset	equ	7012h
SFXPlayPreset	equ	7013h
SFXStopAll	equ	7014h
SFXBuffer	equ	7020h

;
; Standard cartridge header
;
    org    HEADER_OFS
    db     " LICENSED BY SNK CORPORATION"    ; 28 bytes license string
    dd     _start                            ; Program Counter
    dw     0                                 ; Catalog number
    db     0                                 ; Sub catalog number
    db     10h                               ; colour or b+w (10h = colour)
;           123456789012
    db     "            "                    ; Game name (12 bytes)
    dd     0,0,0,0                           ; padding - reserve for future use

; User Interrupt Vectors

UserIntVect:
        dd      nada            ; Software Interrupt (SWI 3)
        dd      nada            ; Software Interrupt (SWI 4)
        dd      nada            ; Software Interrupt (SWI 5)
        dd      nada            ; Software Interrupt (SWI 6)
        dd      nada            ; RTC Alarm Interrupt
        dd      VBlankInt       ; Vertical Blanking Interrupt
        dd      nada            ; Interrupt from Z80
        dd      nada            ; Timer Interrupt (8 bit timer 0)
        dd      nada            ; Timer Interrupt (8 bit timer 1)
        dd      nada            ; Timer Interrupt (8 bit timer 2)
        dd      nada            ; Timer Interrupt (8 bit timer 3)
        dd      nada            ; Serial Transmission Interrupt
        dd      nada            ; Serial Reception Interrupt
        dd      nada            ; (Reserved)
        dd      nada            ; End Micro DMA Int (MicroDMA 0)
        dd      nada            ; End Micro DMA Int (MicroDMA 1)
        dd      nada            ; End Micro DMA Int (MicroDMA 2)
        dd      nada            ; End Micro DMA Int (MicroDMA 3)

nada:   reti

; *** Start of User Code ***

_start:
        calr    OS_VERSION      ; Initialize NGP or NGPC mode

        set     6,(rUSERA)      ; User Answer

; Install User Interrupt Vectors

        lda     xix,(UserIntVect)
        lda     xiy,(rSWI3)
        ld      b,18
UIVloop:
        ld      xwa,(xix+)
        ld      (xiy+),xwa
        djnz    b,UIVloop

; set up screen size 160x152
    ld   (08002h),0
    ld   (08003h),0
    ld   (08004h),0a0h
    ld   (08005h),98h

    ldw (_BGCPAL), 0
    ld (8118h), 80h

    calr InitialiseMemory

; copy internal charset to character RAM
    ld      ra3,3
    ld      rw3,VECT_SYSFONTSET
    calr    SYSTEM_CALL

; copy palette to character palette RAM
    ld bc, (ENDPALETTE-PALETTE)/2
    ld xde, _SCR1PAL
    ld xhl, PALETTE
    ldirw (xde+),(xhl+)


; enable interrupts
    ei

    calr InstallSoundDriver

	calr ScreenSetup
	ld wa, 1 ; cursor on, position 0
	calr DrawCursor

_main:

    ld a, (rSL)
    
    bit PADB_UP, a
    call nz, MoveUp

    bit PADB_DOWN, a
    call nz, MoveDown

    bit PADB_LEFT, a
    call nz, MoveLeft

    bit PADB_RIGHT, a
    call nz, MoveRight

    bit PADB_B, a
    call nz, DoSelect

    bit PADB_A, a
    call nz, DoPlay

    ld w, 5
    calr vbWait


; User shutdown? (Power off pressed?)

        cp      (rUSERS),0
        jp      z,_main      ; no

; Power off NGP

        ld      rw3,VECT_SHUTDOWN
        calr    SYSTEM_CALL

_done:   jr      _done



MoveUp:
	push xbc
	push xde
	push xhl
	ld xbc, 0
	ld c, (CurPos)
	sll xbc
	ld xhl, xbc
	add xbc, Channel ; first value
	ldw de, (xbc)
	add de, 32
	add xhl, ValueLimits
	and de, (xhl)
	ldw (xbc), de
	ld w, (CurPos)
	calr DrawValue
	pop xhl
	pop xde
	pop xbc
	ret

MoveDown:
	push xbc
	push xde
	push xhl
	ld xbc, 0
	ld c, (CurPos)
	sll xbc
	ld xhl, xbc
	add xbc, Channel
	ldw de, (xbc)
	sub de, 32
	add xhl, ValueLimits
	and de, (xhl)
	ldw (xbc), de
	ld w, (CurPos)
	calr DrawValue
	pop xhl
	pop xde
	pop xbc
	ret

MoveLeft:
	push xbc
	push xde
	push xhl
	ld xbc, 0
	ld c, (CurPos)
	sll xbc
	ld xhl, xbc
	add xbc, Channel
	ldw de, (xbc)
	sub de, 1
	add xhl, ValueLimits
	and de, (xhl)
; special cases
   cp (CurPos), 0 ; channel 0-4
   jr nz, ML_SC_1
   cp de, 7
   jr nz, ML_SC_1
   ld de, 4
   jr ML_SC_End
ML_SC_1:
   cp (CurPos), 6 ;  Tone OWB
   jr nz, ML_SC_2
   cp de, 3
   jr nz, ML_SC_2
   ld de, 2
   jr ML_SC_End
ML_SC_2:
   cp (CurPos), 12 ;  Vol OWB
   jr nz, ML_SC_End
   cp de, 3
   jr nz, ML_SC_End
   ld de, 2
ML_SC_End:
	ldw (xbc), de
	ld w, (CurPos)
	calr DrawValue
	pop xhl
	pop xde
	pop xbc
	ret

MoveRight:
	push xbc
	push xde
	push xhl
	ld xbc, 0
	ld c, (CurPos)
	sll xbc
	ld xhl, xbc
	add xbc, Channel
	ldw de, (xbc)
	add de, 1
	add xhl, ValueLimits
	and de, (xhl)
; special cases
   cp (CurPos), 0 ; channel 0-4
   jr nz, MR_SC_1
   cp de, 5
   jr nz, MR_SC_1
   ld de, 0
   jr MR_SC_End
MR_SC_1:
   cp (CurPos), 6 ;  Tone OWB
   jr nz, MR_SC_2
   cp de, 3
   jr nz, MR_SC_2
   ld de, 0
   jr MR_SC_End
MR_SC_2:
   cp (CurPos), 12 ;  Vol OWB
   jr nz, MR_SC_End
   cp de, 3
   jr nz, MR_SC_End
   ld de, 0
MR_SC_End:
	ldw (xbc), de
	ld w, (CurPos)
	calr DrawValue
	pop xhl
	pop xde
	pop xbc
	ret


DoSelect:
	push wa
	; erase cursor
	ld w, (CurPos)
	ld a, 0
	calr DrawCursor

	inc (CurPos)
	cp (CurPos), MAX_VARIABLES
	jr nz, DoS_1

	; wrap
	ld (CurPos), 0
DoS_1:	
	; draw new cursor
	ld w, (CurPos)
	ld a, 1
	calr DrawCursor
	pop wa
	ret

; ---------------------------------------------
; Play button has been pressed. Copy the values
; to the buffer and instruct the driver to play
; ---------------------------------------------
DoPlay:
   push a
   push xde

   ; wait for button release
DoPlayWait:
   ld a, (rSL)
   bit PADB_A, a
   jr nz, DoPlayWait

   ; copy data to z80 ram
   ld xde, SFXBuffer
   ld a, (Channel)
   ld (xde+), a
   ld a, (Length)
   ld (xde+), a
   ld a, (Repeat)
   ld (xde+), a

   ldw wa, (Init_Tone)
   ldw (xde+), wa
   ldw wa, (Tone_Step)
   ldw (xde+), wa
   ld a, (Tone_Speed)
   ld (xde+), a
   ld a, (Tone_OWB)
   ld (xde+), a
   ldw wa, (Tone_LLimit)
   ldw (xde+), wa
   ldw wa, (Tone_ULimit)
   ldw (xde+), wa

   ld a, (Init_Vol)
   ld (xde+), a
   ld a, (Vol_Step)
   ld (xde+), a
   ld a, (Vol_Speed)
   ld (xde+), a
   ld a, (Vol_OWB)
   ld (xde+), a
   ld a, (Vol_LLimit)
   ld (xde+), a
   ld a, (Vol_ULimit)
   ld (xde+), a

   ; instruct to play
   ld (SFXPlayBuffer), 1
   jp PlayEnd

PlayEnd:
   pop xde
   pop a
   ret
	


; ---------------------------------------------------------------
; DrawCursor - Takes 'w' as the value index and displays it at
; predefined y positions. if 'a' is 0 then cursor is cleared
; else it is drawn
; ---------------------------------------------------------------
DrawCursor:
	push xwa
	push xbc
	push xde
	ld xbc, 0
	ld c, w
	
	ld xde, ValueYPos
	add xde, xbc

	ld l, (xde)
	ld h, 11
	ld w, 0
	
	cp a, 0
	jr z, DC_1
	
	; draw cursor
	ld a, '>'
	jr DC_2

DC_1:
	; clear cursor
	ld a, ' '

DC_2:	
	calr DrawCharFront
	pop xde
	pop xbc
	pop xwa
	ret


strHex	db "0123456789ABCDEF"



;---------------------------------------------------------
; Display an 8bit hex number (e.g. FF) held in 'a' at
; position xy = hl
;---------------------------------------------------------
DisplayHex8:
	push xwa
	push xbc
	push xde
	push xhl

	ld w, 0
	ld xbc, 0
	ld c, a
	and c, 0f0h	
	srl 4, c
	ld xde, strHex
	add xde, xbc
	push a
	ld a, (xde)
	calr DrawCharFront
	pop a
	ld xbc, 0
	ld c, a
	and c, 0fh
	ld xde, strHex
	add xde, xbc
	ld a, (xde)
	inc h
	calr DrawCharFront
	
	pop xhl
	pop xde
	pop xbc
	pop xwa
	ret

; ------------------------------------------------------
; Uses DisplayHex8 to display 2 bytes of hex (e.g. FFFF)
; inputs : wa = hex values, hl = xy
; ------------------------------------------------------
DisplayHex16:
	push hl
	push wa
	ld a,w
	calr DisplayHex8
	pop wa
	add h, 2
	calr DisplayHex8
	pop hl
	ret
	

; ---------------------------------------------------------------
; DrawValue - Takes 'w' as the value index and displays it at
; specified x & y positions
; ---------------------------------------------------------------
DrawValue:
	push xwa
	push xbc
	push xde
	push xhl

	; get xpos
	ld h, 12

	; get Y Pos
	ld xbc, 0
	ld c, w
	ld xde, ValueYPos
	add xde, xbc
	ld l, (xde)

	; get initial value
	sll xbc
	add xbc, Channel

	; get number of bytes to display
	ld xde, 0
	ld e, w
	add xde, ValueBytes

	; get actual value
	ldw wa, (xbc)

	cp (xde), 1
	jr nz, DV_1
	; draw a single hex value
	calr DisplayHex8
	jr DV_2
DV_1:
	calr DisplayHex16
DV_2:	
	pop xhl
	pop xde
	pop xbc
	pop xwa
	ret

ValueBytes	db 1,1,1,2,2,1,1,2,2,1,1,1,1,1,1
ValueYPos:      db 1,2,3,5,6,7,8,9,10,12,13,14,15,16,17
ValueLimits:	dw 7,255,255,1023,1023,255,3,1023,1023
                dw 15,15,255,3,15,15

strValues   db "  CHANNEL"
            db "   LENGTH"
	    db "  REPEATS"
            db "INIT TONE"
            db "     STEP"
            db "    SPEED"
            db "  OFF/W/B"
            db "LOW LIMIT"
            db "UPR LIMIT"
            db " INIT VOL"
            db "     STEP"
            db "    SPEED"
            db "  OFF/W/B"
            db "LOW LIMIT"
            db "UPR LIMIT"


ScreenSetup
   ld w, 0
   ld xix, 0

   ld xde, strValues
SS_1:
   ld xhl, ValueYPos
   add xhl, xix
   ld l, (xhl)
   ld h, 1
   calr RenderString
   calr DrawValue
   add xde, 9 ; next string
   inc xix
   inc w
   cp xix, MAX_VARIABLES
   jr nz, SS_1
   ret



; ---------------------------------------------------------------
; Install Sound Driver
;
; ---------------------------------------------------------------
InstallSoundDriver:
	ldw (00b8h), 0aaaah ; stop the z80 and sound chip

	; copy the driver into memory
	ld bc, SoundDriverEnd - SoundDriver
	ld xde, 7000h ; where the z80 shared memory is
	ld xhl, SoundDriver
	ldir (xde+), (xhl+)

	ldw (00b8h), 05555h ; start the z80 and sound chip
	ret
	
SoundDriver:
	binclude "sfxdriv.z80"
SoundDriverEnd:

; ---------------------------------------------------------------
; Initialise Memory before game starts
;
; ---------------------------------------------------------------
InitialiseMemory:
; Clear Sprite Attribute Memory

        ld      wa,0
        ld      b,64
        ld      xix,_SPRRAM
SAMloop:
        ld      (xix+),wa
        ld      (xix+),wa
        djnz    b,SAMloop

; Clear RAM
        ld      b, 255
        ld      xix, _MAINRAM
RAMLoop: 
        ld      (xix+), 0
        djnz    b, RAMLoop

; Clear Scroll 1 & 2 VRAM with 0
ClearScreen:
        ld      xhl,_SCR1RAM
        ld      wa, 0
        ld      (xhl),wa
        ld      xde,xhl
        inc     xde
        inc     xde
        ld      bc,800h-1
        ldirw
        ret





; ---------------------------------------------------------------
;
; RenderNumber
; Draw a given num of a character number on the screen.
; inputs hl=xy, wa=number, c=number of char (e.g. 364=3)
;        b = palette number
;
; ---------------------------------------------------------------
RenderNumber:
   push xwa
   push xbc
   push xix
   push xhl
   and xwa, 0ffffh
   add b,b
   add h,c
rnAgain:
   dec h
   div xwa, 10
   ld xix, 0
   ld ix, wa
   srl 16, xwa
   add xwa, '0'
   ; add palette 
   add w,b
   calr DrawCharFront
   ld xwa, xix
   dec c
   jr nz, rnAgain
   pop xhl
   pop xix
   pop xbc
   pop xwa
   ret

; ---------------------------------------------------------------
; 
; DrawCharFront
;
; Draws a character on the foreground plane
; inputs: wa = char, hl = xy
;
; ---------------------------------------------------------------
DrawCharFront
    push xhl
    push xde
    push xbc
    ld xde, _SCR1RAM
    ld c, h
    ld b, 0
    mul xbc, 2
    add de, bc
    ld h, 0
    mul xhl, 40h
    add de, hl
    ld (xde), a
    add xde, 1
    ld (xde), w
    pop xbc
    pop xde
    pop xhl
    ret

; ---------------------------------------------------------------
;
; RenderValue
; Draws text on the screen (front plane)
; inputs: hl=xy, xde=pointer 9 char string
;
; ---------------------------------------------------------------
RenderString:
    push xwa
    push xbc
    push xde
    push xhl
    ld c, 0
rsAgain:
    ld xwa, 0
    ld a, (xde)
    calr DrawCharFront
    inc h
    inc xde
    inc c
    cp c, 9
    jr nz, rsAgain
rsEnd:
    pop xhl
    pop xde
    pop xbc
    pop xwa
    ret


; ---------------------------------------------------------------
;
; vbWait
; Waits for n vertical blank interrupts
; inputs: w = number of VBIs to wait for
;
; ---------------------------------------------------------------
vbWait:
   push xwa
   push xbc
   ld (VBCOUNTER), 0
   ld c, w
vbw1:
   ld w, c
   ld a, (VBCOUNTER)
   sub w, a
   jr nc, vbw1
   pop xbc
   pop xwa
   ret



VBlankInt:
    ; Update Watch Dog Timer to prevent CPU reset
    ld      (rWDCR),WD_CLR

    ; increment a counter
    inc (VBCOUNTER)
    inc (SfxPulse)
    reti


PALETTE
	dw 00000h, 00fffh, 00fffh, 00fffh
ENDPALETTE


SYSTEM_CALL:
        SystemCallCode
OS_VERSION:
        OsVersionCode

    ; make the cartridge size correct
    org HEADER_OFS + 07FFFh
    db 0FFh

